#!/usr/bin/env python3

import sys, re, os, uuid

from datetime import *

from GoldenChild.xpath import *
from Perdy.pretty import *


#________________________________________________________________
def generateID():
	"""
	todo: cache uuid based on package path Namespac
	"""
	return 'EDDO_%s' % uuid.uuid4()


#________________________________________________________________
class XMI:

	def __init__(self):
		self.exporter = '$HeadURL: git@github.com:eddo888/Tools/xmi.py $'
		self.version = '$Revision: 11983 $'
		self.timestamp = '%Y-%m-%d %H:%M:%S'

		self.classDiagramStyle = {
			'ShowPrivate': '1',
			'ShowProtected': '1',
			'ShowPublic': '1',
			'HideRelationships': '0',
			'Locked': '0',
			'Border': '1',
			'HighlightForeign': '1',
			'PackageContents': '1',
			'SequenceNotes': '0',
			'ScalePrintImage': '1',
			'PPgs.cx': '1',
			'PPgs.cy': '1',
			'DocSize.cx': '1000',
			'DocSize.cy': '1000',
			'ShowDetails': '0',
			'Orientation': 'P',
			'Zoom': '100',
			'ShowTags': '0',
			'OpParams': '1',
			'VisibleAttributeDetail': '0',
			'ShowOpRetType': '1',
			'ShowIcons': '1',
			'CollabNums': '1',
			'HideProps': '1',
			'ShowReqs': '0',
			'ShowCons': '0',
			'PaperSize': '1',
			'HideParents': '1',
			'UseAlias': '0',
			'HideAtts': '0',
			'HideOps': '0',
			'HideStereo': '0',
			'HideElemStereo': '0',
			'ShowTests': '0',
			'ShowMaint': '0',
			'ConnectorNotation': 'UML 2.1',
			'ExplicitNavigability': '0',
			'AdvancedElementProps': '1',
			'AdvancedFeatureProps': '1',
			'AdvancedConnectorProps': '1',
			'ShowNotes': '0',
			'SuppressBrackets': '0',
			'SuppConnectorLabels': '0',
			'PrintPageHeadFoot': '0',
			'ShowAsList': '0',
		}

		self.activityDiagramStyle = {
			'ShowPrivate': '1',
			'ShowProtected': '1',
			'ShowPublic': '1',
			'HideRelationships': '0',
			'Locked': '0',
			'Border': '1',
			'HighlightForeign': '1',
			'PackageContents': '1',
			'SequenceNotes': '0',
			'ScalePrintImage': '0',
			'PPgs.cx': '1',
			'PPgs.cy': '1',
			'DocSize.cx': '850',
			'DocSize.cy': '1098',
			'ShowDetails': '0',
			'Orientation': 'P',
			'Zoom': '100',
			'ShowTags': '0',
			'OpParams': '1',
			'VisibleAttributeDetail': '0',
			'ShowOpRetType': '1',
			'ShowIcons': '1',
			'CollabNums': '0',
			'HideProps': '0',
			'ShowReqs': '0',
			'ShowCons': '0',
			'PaperSize': '1',
			'HideParents': '0',
			'UseAlias': '0',
			'HideAtts': '0',
			'HideOps': '0',
			'HideStereo': '0',
			'HideElemStereo': '0',
			'ShowTests': '0',
			'ShowMaint': '0',
			'ConnectorNotation': 'UML 2.1',
			'ExplicitNavigability': '0',
			'AdvancedElementProps': '1',
			'AdvancedFeatureProps': '1',
			'AdvancedConnectorProps': '1',
			'ShowNotes': '0',
			'SuppressBrackets': '0',
			'SuppConnectorLabels': '0',
			'PrintPageHeadFoot': '0',
			'ShowAsList': '0',
		}

		self.requirementsDiagramStyle = {
			'ShowPrivate': '1',
			'ShowProtected': '1',
			'ShowPublic': '1',
			'HideRelationships': '0',
			'Locked': '0',
			'Border': '1',
			'HighlightForeign': '1',
			'PackageContents': '1',
			'SequenceNotes': '0',
			'ScalePrintImage': '0',
			'PPgs.cx': '1',
			'PPgs.cy': '1',
			'DocSize.cx': '815',
			'DocSize.cy': '1067',
			'ShowDetails': '0',
			'Orientation': 'P',
			'Zoom': '100',
			'ShowTags': '0',
			'OpParams': '1',
			'VisibleAttributeDetail': '0',
			'ShowOpRetType': '1',
			'ShowIcons': '1',
			'CollabNums': '0',
			'HideProps': '0',
			'ShowReqs': '0',
			'ShowCons': '0',
			'PaperSize': '32512',
			'HideParents': '0',
			'UseAlias': '0',
			'HideAtts': '0',
			'HideOps': '0',
			'HideStereo': '0',
			'HideElemStereo': '0',
			'ShowTests': '0',
			'ShowMaint': '0',
			'ConnectorNotation': 'UML 2.1',
			'ExplicitNavigability': '0',
			'AdvancedElementProps': '1',
			'AdvancedFeatureProps': '1',
			'AdvancedConnectorProps': '1',
			'ShowNotes': '0',
			'SuppressBrackets': '0',
			'SuppConnectorLabels': '0',
			'PrintPageHeadFoot': '0',
			'ShowAsList': '1',
		}

		self.namespaces = {'UML': 'omg.org/UML1.3'}

		self.now = datetime.now()
		self.makeDocument()
		self.makeModel()
		self.makeExtensions()
		self.stereotypes = {}
		return

	def makeDocument(self):
		(self.doc, self.ctx, self.nsp) = getContextFromString('<XMI/>')
		for prefix in list(self.namespaces.keys()):
			self.ctx.xpathRegisterNs(prefix, self.namespaces[prefix])
		# version
		self.root = self.doc.getRootElement()
		for prefix in list(self.namespaces.keys()):
			self.root.setProp('xmlns:%s' % prefix, self.namespaces[prefix])
		self.root.setProp('xmi.version', '1.1')
		# timestamp
		self.root.setProp('timestamp', self.now.strftime(self.timestamp))
		# header
		header = addElement(self.doc, 'XMI.header', self.root)
		# documentation
		doco = addElement(self.doc, 'XMI.documentation', header)
		addElementText(self.doc, 'XMI.exporter', self.exporter, doco)
		addElementText(self.doc, 'XMI.exporterVersion', self.version, doco)
		# metamodel
		meta = addElement(self.doc, 'XMI.metamodel', header)
		meta.setProp('xmi.name', 'UML')
		meta.setProp('xmi.version', '1.4')
		return

	def makeModel(self):
		self.content = addElement(self.doc, 'XMI.content', self.root)
		model = addElement(self.doc, 'UML:Model', self.content)
		model.setProp('isAbstract', 'false')
		model.setProp('isLeaf', 'false')
		model.setProp('isRoot', 'true')
		model.setProp('isSpecification', 'false')
		model.setProp('name', self.now.strftime(self.timestamp))
		model.setProp('visibility', 'public')
		model.setProp('xmi.id', generateID())
		self.modelNS = addElement(self.doc, 'UML:Namespace.ownedElement', model)
		#self.rootClass = self.makeClass('EARootClass',self.modelNS)
		#self.rootClass.parent.setProp('isRoot','true')
		return

	def makeExtensions(self):
		self.extensions = addElement(self.doc, 'XMI.extensions')
		self.extensions.setProp('xmi.extender', 'Enterprise Architect 2.5')
		self.files = addElement(self.doc, 'EAModel.file', self.extensions)
		return

	def makePackage(self, name, parent):
		package = addElement(self.doc, 'UML:Package', parent)
		package.setProp('isAbstract', 'false')
		package.setProp('isLeaf', 'false')
		package.setProp('isRoot', 'true')
		package.setProp('isSpecification', 'false')
		package.setProp('name', name)
		package.setProp('visibility', 'public')
		package.setProp('xmi.id', generateID())
		element = addElement(self.doc, 'UML:Namespace.ownedElement', package)
		return element

	def getPackage(self, name, parent):
		package = getElement(self.ctx, '*[@name="%s"]' % name, parent)
		if package == None:
			package = self.makePackage(name, parent)
		return package

	def makeCollaboration(self, name, parent, uid=None):
		if not uid: uid = generateID()
		collaboration = addElement(self.doc, 'UML:Collaboration', parent)
		collaboration.setProp('name', name)
		collaboration.setProp('xmi.id', uid)
		return addElement(self.doc, 'UML:Namespace.ownedElement', collaboration)

	def makeRequirement(self, name, parent, uid=None):
		if not uid: uid = generateID()
		requirement = addElement(self.doc, 'UML:ClassifierRole', parent)
		requirement.setProp('isAbstract', 'false')
		requirement.setProp('isLeaf', 'false')
		requirement.setProp('isRoot', 'true')
		requirement.setProp('isSpecification', 'false')
		requirement.setProp('name', name)
		requirement.setProp('visibility', 'public')
		requirement.setProp('xmi.id', uid)
		self.makeLocalTag('stereotype', 'Functional', requirement)
		self.makeLocalTag('ea_eleType', 'element', requirement)
		self.makeLocalTag('ea_stype', 'Requirement', requirement)
		self.makeLocalTag('ea_ntype', '0', requirement)
		return addElement(self.doc, 'UML:Classifier.feature', requirement)

	def makeActor(self, name, parent, uid=None):
		if not uid: uid = generateID()
		actor = addElement(self.doc, 'UML:Actor', parent)
		actor.setProp('isAbstract', 'false')
		actor.setProp('isLeaf', 'false')
		actor.setProp('isRoot', 'true')
		actor.setProp('isSpecification', 'false')
		actor.setProp('name', name)
		actor.setProp('visibility', 'public')
		actor.setProp('xmi.id', uid)
		self.makeLocalTag('ea_eleType', 'element', actor)
		self.makeLocalTag('ea_stype', 'Actor', actor)
		self.makeLocalTag('ea_ntype', '0', actor)
		return addElement(self.doc, 'UML:Classifier.feature', actor)

	def makeUseCase(self, name, parent, uid=None):
		if not uid: uid = generateID()
		useCase = addElement(self.doc, 'UML:UseCase', parent)
		useCase.setProp('isAbstract', 'false')
		useCase.setProp('isLeaf', 'false')
		useCase.setProp('isRoot', 'true')
		useCase.setProp('isSpecification', 'false')
		useCase.setProp('name', name)
		useCase.setProp('visibility', 'public')
		useCase.setProp('xmi.id', uid)
		self.makeLocalTag('ea_eleType', 'element', useCase)
		self.makeLocalTag('ea_stype', 'UseCase', useCase)
		self.makeLocalTag('ea_ntype', '0', useCase)
		return addElement(self.doc, 'UML:Classifier.feature', useCase)

	def makeComponent(self, name, parent, uid=None):
		if not uid: uid = generateID()
		system = addElement(self.doc, 'UML:Component', parent)
		system.setProp('isAbstract', 'false')
		system.setProp('isLeaf', 'false')
		system.setProp('isRoot', 'true')
		system.setProp('isSpecification', 'false')
		system.setProp('name', name)
		system.setProp('visibility', 'public')
		system.setProp('xmi.id', uid)
		self.makeLocalTag('complexity', '1', system)
		self.makeLocalTag('ea_eleType', 'element', system)
		self.makeLocalTag('ea_stype', 'Component', system)
		self.makeLocalTag('ea_ntype', '0', system)
		return addElement(self.doc, 'UML:Classifier.feature', system)

	def makeClass(self, name, parent, uid=None):
		if not uid: uid = generateID()
		clasz = addElement(self.doc, 'UML:Class', parent)
		clasz.setProp('isAbstract', 'false')
		clasz.setProp('isLeaf', 'false')
		clasz.setProp('isRoot', 'true')
		clasz.setProp('isSpecification', 'false')
		clasz.setProp('name', name)
		clasz.setProp('visibility', 'public')
		clasz.setProp('xmi.id', uid)
		self.makeLocalTag('ea_eleType', 'element', clasz)
		return addElement(self.doc, 'UML:Classifier.feature', clasz)

	def assignBaseClass(self, subtype, supertype, parent, uid=None):
		if not uid: uid = generateID()
		generalized = addElement(self.doc, 'UML:Generalization', parent)
		generalized.setProp('subtype', supertype.parent.prop('xmi.id'))
		generalized.setProp('supertype', subtype.parent.prop('xmi.id'))
		generalized.setProp('visibility', 'public')
		generalized.setProp('xmi.id', uid)
		return

	def makeAttribute(self, name, type, value, parent, tid=None, array=False):
		attribute = addElement(self.doc, 'UML:Attribute', parent)
		attribute.setProp('name', name)
		attribute.setProp('visibility', 'private')
		if array:
			self.makeLocalTag('collection', 'true', attribute)
			self.makeLocalTag('container', '[]', attribute)
		if type:
			tid = type.parent.prop('xmi.id')
		if tid:
			sf = addElement(self.doc, 'UML:StructuralFeature.type', attribute)
			classifier = addElement(self.doc, 'UML:Classifier', sf)
			classifier.setProp('xmi.idref', tid)
		if value:
			av = addElement(self.doc, 'UML:Attribute.initialValue', attribute)
			ue = addElement(self.doc, 'UML:Expression', av)
			ue.setProp('body', value)
		return attribute

	def makeOperation(self, name, parameters, returns, parent, pids={}, rid=None):
		operation = addElement(self.doc, 'UML:Operation', parent)
		operation.setProp('name', name)
		operation.setProp('visibility', 'public')
		bf = addElement(self.doc, 'UML:BehavioralFeature.parameter', operation)
		if parameters:
			pids = {}
			for p in list(parameters.keys()):
				pids[p] = parameters[p].parent.prop('xmi.id')
		for p in list(pids.keys()):
			self.makeParameter(p, None, 'in', bf, tid=pids[p])
		self.makeParameter('return', returns, 'return', bf, tid=rid)
		return

	def makeParameter(self, name, type, kind, parent, tid=None):
		if type:
			tid = type.parent.prop('xmi.id')
		if not tid:
			return
		parameterP = addElement(self.doc, 'UML:Parameter', parent)
		parameterP.setProp('xmi.id', generateID())
		parameterP.setProp('name', name)
		parameterP.setProp('kind', kind)
		parameterT = addElement(self.doc, 'UML:Parameter.type', parameterP)
		parameterC = addElement(self.doc, 'UML:Class', parameterT)
		parameterC.setProp('xmi.idref', tid)
		return

	def makeAssociation(self,
																					name,
																					source,
																					target,
																					parent,
																					sid=None,
																					tid=None,
																					array=False):
		if source:
			sid = source.parent.prop('xmi.id')
		if target:
			tid = target.parent.prop('xmi.id')
		association = addElement(self.doc, 'UML:Association', parent)
		association.setProp('name', name)
		association.setProp('xmi.id', generateID())
		association.setProp('visibility', 'public')
		association.setProp('isAbstract', 'false')
		association.setProp('isLeaf', 'false')
		association.setProp('isRoot', 'false')
		# connect em
		connection = addElement(self.doc, 'UML:Association.connection', association)
		# source
		sourceEnd = addElement(self.doc, 'UML:AssociationEnd', connection)
		sourceEnd.setProp('aggregation', 'none')
		sourceEnd.setProp('isNavigable', 'false')
		sourceEnd.setProp('xmi.id', generateID())
		sourceParticipant = addElement(self.doc, 'UML:AssociationEnd.participant',
																																	sourceEnd)
		sourceClass = addElement(self.doc, 'UML:Class', sourceParticipant)
		sourceClass.setProp('xmi.idref', sid)
		#target
		targetEnd = addElement(self.doc, 'UML:AssociationEnd', connection)
		targetEnd.setProp('aggregation', 'none')
		targetEnd.setProp('isNavigable', 'true')
		targetEnd.setProp('xmi.id', generateID())
		if array:
			targetEndMulti = addElement(self.doc, 'UML:AssociationEnd.multiplicity',
																															targetEnd)
			multi = addElement(self.doc, 'UML:Multiplicity', targetEndMulti)
			multi.setProp('xmi.id', generateID())
			multiRangeP = addElement(self.doc, 'UML:Multiplicity.range', multi)
			multiRangeC = addElement(self.doc, 'UML:MultiplicityRange', multiRangeP)
			multiRangeC.setProp('xmi.id', generateID())
			multiRangeC.setProp('lower', '0')
			multiRangeC.setProp('upper', '-1')
		targetParticipant = addElement(self.doc, 'UML:AssociationEnd.participant',
																																	targetEnd)
		targetClass = addElement(self.doc, 'UML:Class', targetParticipant)
		targetClass.setProp('xmi.idref', tid)
		return association

	def makeUsage(self,
															name,
															source,
															target,
															parent,
															sid=None,
															tid=None,
															array=False):
		if source:
			sid = source.parent.prop('xmi.id')
		if target:
			tid = target.parent.prop('xmi.id')
		association = addElement(self.doc, 'UML:Association', parent)
		association.setProp('name', name)
		association.setProp('xmi.id', generateID())
		association.setProp('visibility', 'public')
		association.setProp('isAbstract', 'false')
		association.setProp('isLeaf', 'false')
		association.setProp('isRoot', 'false')
		self.makeLocalTag('ea_type', 'Usage', association)

		# connect em
		connection = addElement(self.doc, 'UML:Association.connection', association)
		# source
		sourceEnd = addElement(self.doc, 'UML:AssociationEnd', connection)
		sourceEnd.setProp('aggregation', 'none')
		sourceEnd.setProp('isNavigable', 'false')
		sourceEnd.setProp('type', sid)
		self.makeLocalTag('ea_end', 'source', sourceEnd)
		#target
		targetEnd = addElement(self.doc, 'UML:AssociationEnd', connection)
		targetEnd.setProp('aggregation', 'none')
		targetEnd.setProp('isNavigable', 'true')
		targetEnd.setProp('type', tid)
		self.makeLocalTag('ea_end', 'target', targetEnd)
		return association

	def makeObject(self, name, type, parent):
		cr = addElement(self.doc, 'UML:ClassifierRole', parent)
		cr.setProp('name', name)
		cr.setProp('visibility', 'public')
		cr.setProp('xmi.id', generateID())
		#cr.setProp('base',self.rootClass.parent.prop('xmi.id'))
		self.makeLocalTag('classifier', type.parent.prop('xmi.id'), cr)
		self.makeLocalTag('classname', type.parent.prop('name'), cr)
		self.makeLocalTag('ea_stype', 'Object', cr)
		return cr

	def makeComment(self, text, parent):
		return

	def makeEAFile(self, path, type, parent):
		flement = addElement(self.doc, 'EAFile', self.files)
		flement.setProp('name', path)
		flement.setProp('type', type)
		flement.setProp('subject', parent.prop('xmi.id'))
		if type == 'Web Address':
			return
		if type == 'Local File':
			self.makeLocalTag('filesize', '%d' % os.path.getsize(path), flement)
			modified = datetime.fromtimestamp(os.path.getmtime(path))
			self.makeLocalTag('timestamp', modified.strftime(self.timestamp), flement)
		return

	def addDocumentation(self, text, parent):
		self.makeLocalTag('documentation', text, parent.parent)

	def makeLocalTag(self, name, value, parent):
		tags = getElement(
			self.ctx, '*[contains(local-name(),"taggedValue")]', parent) or addElement(
				self.doc, 'UML:ModelElement.taggedValue', parent)
		tag = getElement(self.ctx, '*[@tag="%s"]' % name, tags) or addElement(
			self.doc, 'UML:TaggedValue', tags)
		tag.setProp('tag', name)
		tag.setProp('value', value)
		return

	def makeContentTag(self, name, value, parent):
		tag = addElement(self.doc, 'UML:TaggedValue', self.content)
		tag.setProp('xmi.id', generateID())
		tag.setProp('tag', name)
		tag.setProp('value', value)
		tag.setProp('modelElement', parent.parent.prop('xmi.id'))
		return

	def makeStereotype(self, name, parent):
		if not name in list(self.stereotypes.keys()):
			ust = addElement(self.doc, 'UML:Stereotype', self.modelNS)
			ust.setProp('name', name)
			ust.setProp('xmi.id', generateID())
			self.stereotypes[name] = ust
		mes = addElement(self.doc, 'UML:ModelElement.stereotype', parent.parent)
		stereotype = addElement(self.doc, 'UML:Stereotype', mes)
		stereotype.setProp('xmi.idref', self.stereotypes[name].prop('xmi.id'))
		return

	def makeActivityModel(self, name, parent):
		model = addElement(self.doc, 'UML:ActivityGraph', parent)
		model.setProp('name', name)
		model.setProp('xmi.id', generateID())
		return model

	def makeTransition(self, source, target, parent, name=None):
		transitions = getElement(self.ctx, '*[contains(local-name(),"transitions")]',
																											parent)
		if transitions is None:
			transitions = addElement(self.doc, 'UML:StateMachine.transitions', parent)
		transition = addElement(self.doc, 'UML:Transition', transitions)
		transition.setProp('xmi.id', generateID())
		if name != None:
			transition.setProp('name', name)

		sourceT = addElement(self.doc, 'UML:Transition.source', transition)
		sourceS = addElement(self.doc, source.name, sourceT)
		sourceS.setProp('xmi.idref', source.prop('xmi.id'))

		targetT = addElement(self.doc, 'UML:Transition.target', transition)
		targetS = addElement(self.doc, target.name, targetT)
		targetS.setProp('xmi.idref', target.prop('xmi.id'))

		return transition

	def makeActivitySwimLane(self, name, parent):
		partitions = getElement(self.ctx, '*[contains(local-name(),"partition")]',
																										parent)
		if partitions is None:
			partitions = addElement(self.doc, 'UML:ActivityGraph.partition', parent)
		partition = addElement(self.doc, 'UML:Partition', partitions)
		partition.setProp('name', name)
		partition.setProp('xmi.id', generateID())
		return partition

	def addActivityToLane(self, state, parent):
		contents = getElement(self.ctx, '*[contains(local-name(),"contents")]',
																								parent)
		if contents is None:
			contents = addElement(self.doc, 'UML:Partition.contents', parent)
		content = addElement(self.doc, state.name, contents)
		content.setProp('xmi.idref', state.prop('xmi.id'))
		self.makeLocalTag('owner', parent.prop('xmi.id'), state)
		return content

	def makeActivityStartState(self, name, parent, kind=None):
		state = self.makeActivityState(name, 'Pseudostate', parent)
		state.setProp('kind', 'initial')
		return state

	def makeActivityNodeState(self, name, parent):
		state = self.makeActivityState(name, 'ActionState', parent)
		self.makeLocalTag('ea_stype', 'StateNode', state)
		return state

	def makeActivitySwitchState(self, name, parent):
		state = self.makeActivityState(name, 'ActionState', parent)
		state.setProp('kind', 'branch')
		self.makeLocalTag('ea_stype', 'Decision', state)
		return state

	def makeActivityFinishState(self, name, parent, kind=None):
		state = self.makeActivityState(name, 'FinalState', parent)
		return state

	def makeActivityState(self, name, type, parent):
		types = {
			'Pseudostate': 'StateNode',
			'ActionState': 'Action',
			'FinalState': 'Final'
		}
		if type not in list(types.keys()):
			raise Exception('type=%s not in %s' % (type, list(types.keys())))

		top = getElement(self.ctx, '*[contains(local-name(),"top")]', parent)
		if top is None:
			top = addElement(self.doc, 'UML:StateMachine.top', parent)

		cs = getElement(self.ctx, '*[contains(local-name(),"CompositeState")]', top)
		if cs is None:
			cs = addElement(self.doc, 'UML:CompositeState', top)
			cs.setProp('name', 'top')
			cs.setProp('xmi.id', generateID())

		sub = getElement(self.ctx, '*[contains(local-name(),"subvertex")]', cs)
		if sub is None:
			sub = addElement(self.doc, 'UML:CompositeState.subvertex', cs)

		state = addElement(self.doc, 'UML:%s' % type, sub)
		state.setProp('name', name)
		state.setProp('xmi.id', generateID())

		return state

	def makeComponentDiagram(self, name, parent):
		diagram = self.makeDiagram(name, parent)
		diagram.parent.setProp('diagramType', 'ComponentDiagram')
		self.makeLocalTag('type', 'Logical', diagram.parent)
		self.makeLocalTag('EAStyle',
																				self.makeStyle(self.classDiagramStyle), diagram.parent)
		return diagram

	def makeClassDiagram(self, name, parent):
		diagram = self.makeDiagram(name, parent)
		diagram.parent.setProp('diagramType', 'ClassDiagram')
		self.makeLocalTag('type', 'Logical', diagram.parent)
		self.makeLocalTag('EAStyle',
																				self.makeStyle(self.classDiagramStyle), diagram.parent)
		return diagram

	def makeActivityDiagram(self, name, parent):
		diagram = self.makeDiagram(name, parent)
		diagram.parent.setProp('diagramType', 'ActivityDiagram')
		self.makeLocalTag('type', 'Activity', diagram.parent)
		self.makeLocalTag('EAStyle',
																				self.makeStyle(self.activityDiagramStyle), diagram.parent)
		return diagram

	def makeRequirementsDiagram(self, name, parent):
		diagram = self.makeDiagram(name, parent)
		diagram.parent.setProp('diagramType', 'CustomDiagram')
		self.makeLocalTag('type', 'Custom', diagram.parent)
		self.makeLocalTag('EAStyle',
																				self.makeStyle(self.requirementsDiagramStyle),
																				diagram.parent)
		return diagram

	def makeUseCaseDiagram(self, name, parent):
		diagram = self.makeDiagram(name, parent)
		diagram.parent.setProp('diagramType', 'UseCaseDiagram')
		self.makeLocalTag('type', 'Logical', diagram.parent)
		self.makeLocalTag('EAStyle',
																				self.makeStyle(self.classDiagramStyle), diagram.parent)
		return diagram

	def makeDiagram(self, name, parent):
		diagram = addElement(self.doc, 'UML:Diagram', self.content)
		diagram.setProp('name', name)
		diagram.setProp('toolName', 'Enterprise Architect 2.5')
		diagram.setProp('xmi.id', generateID())
		diagram.setProp('owner', parent.parent.prop('xmi.id'))
		self.makeLocalTag('package', parent.parent.prop('xmi.id'), diagram)
		elements = addElement(self.doc, 'UML:Diagram.element', diagram)
		return elements

	def addDiagramClass(self, element, parent, geometry=None, style=None):
		self.addDiagramElement(element.parent, parent, geometry, style)

	def addDiagramState(self, element, parent, geometry=None, style=None):
		self.addDiagramElement(element, parent, geometry, style)

	def addDiagramElement(self, element, parent, geometry=None, style=None):
		child = addElement(self.doc, 'UML:DiagramElement', parent)
		child.setProp('subject', element.prop('xmi.id'))
		if geometry != None:
			child.setProp('geometry', geometry)
		if style != None:
			child.setProp('style', style)
		return child

	def makeStyle(self, types):
		style = ''.join(['%s=%s;' % (x, types[x]) for x in list(types.keys())])
		return style





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, re, xmltodict, argparse, argcomplete, json, vobject, datetime

parser = argparse.ArgumentParser('Convert microsoft .contact to .vcard')

parser.add_argument(
	'-v', '--verbose', action='store_true', help='show detailed output')
parser.add_argument('-o', '--output', action='store', help='output dir')
parser.add_argument('contacts', nargs='*')

argcomplete.autocomplete(parser)
args = parser.parse_args()

if args.verbose:
	sys.stderr.write('args:')
	json.dump(vars(args), sys.stderr, indent=4)
	sys.stderr.write('\n')


def populate(source, name, default=''):
	value = default
	if name in list(source.keys()):
		if '#text' in source[name]:
			value = source[name]['#text']
		else:
			value = source[name]
	return value


def labels(source):
	values = []
	if 'c:LabelCollection' in list(source.keys()):
		if 'c:Label' in source['c:LabelCollection']:
			items = source['c:LabelCollection']['c:Label']
			if type(items) != list:
				items = [items]
			for label in items:
				if type(label) == dict and '#text' in list(label.keys()):
					values.append(label['#text'])
				else:
					values.append(label)
	return values


def convert(source):
	input = open(source)
	contact = xmltodict.parse(input)
	input.close()

	vcard = vobject.vCard()

	name = contact['c:contact']['c:NameCollection']['c:Name']
	vcard.add('n')
	vcard.n.value = vobject.vcard.Name(
		given=populate(name, 'c:GivenName'),
		additional=populate(name, 'c:MiddleName'),
		family=populate(name, 'c:FamilyName'), )

	vcard.add('fn')
	vcard.fn.value = populate(name, 'c:FormattedName')

	#print contact
	position = contact['c:contact']['c:PositionCollection']['c:Position']

	vcard.add('org')
	vcard.org.value = [populate(position, 'c:Company')]

	vcard.add('title')
	vcard.title.value = populate(position, 'c:JobTitle')

	emails = contact['c:contact']['c:EmailAddressCollection']
	for email in emails['c:EmailAddress']:
		ve = vcard.add('email')
		if 'Preferred' in labels(email):
			#ve.type = 'pref'
			None
		ve.type_param = 'internet'
		ve.value = populate(email, 'c:Address')

	addresses = contact['c:contact']['c:PhysicalAddressCollection']
	for address in addresses['c:PhysicalAddress']:
		va = vcard.add('adr')
		for tipe in labels(address):
			#ve = ve.type = tipe
			None
		va.value = vobject.vcard.Address(
			street=populate(address, 'c:Street'),
			city=populate(address, 'c:Locality'),
			code=populate(address, 'c:PostalCode'),
			country=populate(address, 'c:Country'))

	phones = contact['c:contact']['c:PhoneNumberCollection']
	for phone in phones['c:PhoneNumber']:
		vt = vcard.add('tel')
		for tipe in labels(phone):
			#vt.type_param = tipe
			None
		vt.value = populate(phone, 'c:Number')

	vcard.add('note')
	vcard.note.value = contact['c:contact']['c:Notes']['#text']

	vcard.prettyPrint()

	target = '%s.vcf' % source
	if args.output:
		target = '%s/%s' % (args.output, target)

	output = open(target, 'w')
	vcard.serialize(buf=output)
	output.close()
	return


def main():
	if args.output and not os.path.isdir(args.output):
		os.makedirs(args.output)

	for file in args.contacts:
		try:
			convert(file)
		except:
			sys.stderr.write('%s\n' % file)
	return


if __name__ == '__main__': main()





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, json, logging, arrow, codecs

from datetime import datetime
from uuid import uuid4

from Baubles.Logger import Logger
from Perdy.pretty import prettyPrint
from Perdy.parser import printXML
from Perdy.pyxbext import directory
from Argumental.Argue import Argue
from GoldenChild.xpath import *

logger = Logger()
args = Argue()

logger.setLevel(logging.INFO)

#____________________________________________________________
@args.command(single=True)
class COD(object):

	#........................................................
	def __init__(self):
		'''
		setup constructor with lookup lists
		'''
		self.xcolours = {
			'Black': '\033[30m',
			'Red': '\033[31m',
			'Green': '\033[32m',
			'Orange': '\033[33m',
			'Blue': '\033[34m',
			'Purple': '\033[35m',
			'Teal': '\033[36m',
			'White': '\033[37m',
			'Off': '\033[0m',
		}

		self.colours = {
			0: 'White',
			5: 'Blue',
			4: 'Green',
			2: 'Red',
			3: 'Orange',
			6: 'Purple',
		}

		self.xfonts = {
			'bold': '\033[1m',
			'italics': '\033[3m',
			'underline': '\033[4m',
			'strikeout': '\033[9m',
			'normal': '\033[0m',
		}

		self.fonts = {
			0: 'normal',
			8: 'bold',
			2: 'underline',
			1: 'strikeout',
			4: 'italics',
		}



	#........................................................
	@logger.debug
	def __show(self, cod, childItem, indent='', checkboxes=False, shownotes=False):
		'''
		show a single item
		'''
		font = int(getElementText(cod.ctx, 'fontStyle', childItem))
		colour = int(getElementText(cod.ctx, 'color', childItem))
		state = getElementText(cod.ctx, 'completionState', childItem) == 3
		if checkboxes:
			checked = '[x] ' if state else '[ ] '
		else:
			checked = ''

		if colour in self.colours.keys():
			_colour = self.xcolours[self.colours[colour]] or ''
		else:
			_colour = ''
		_font = self.xfonts[self.fonts[font]] or ''
		print('%s%s%s%s%s%s' % (
			indent, _font, _colour, checked,
			getElementText(cod.ctx, 'title', childItem),
			self.xcolours['Off'], )
		)

		if shownotes:
			notes = getElement(cod.ctx, 'notes', childItem)
			if notes:
				note = notes.content
				if note != '(null)':
					print('%s%s  "%s"%s' % (
						self.xcolours['Teal'], '%s  ' % indent
						if checkboxes else indent, note,
						self.xcolours['Off'],
					))

		for grandChild in getElements(cod.ctx, 'ChildItem', childItem):
			self.__show(
				cod,
				grandChild,
				indent='%s  ' % indent,
				checkboxes=checkboxes,
				shownotes=shownotes
			)
			

	#........................................................
	@logger.debug
	def __convert(self, file, cod, checkboxes=False, shownotes=False):
		'''
		show the cod file
		'''
		s = getElementText(cod.ctx, '/Document/Properties/lastModificationTime')
		u = arrow.get(float(s))
		a = u.to('local').format('YYYY-MM-DD HH:mm:ss SSS Z')
		t = getElementText(cod.ctx, '/Document/Properties/title')
		print('%s -> "%s" => %s' % (file, t, a))
		for childItem in getElements(cod.ctx, '/Document/Properties/context/ChildItem'):
			self.__show(cod, childItem, checkboxes=checkboxes, shownotes=shownotes)
			

	#........................................................
	@logger.debug
	@args.operation
	@args.parameter(name='checkboxes', short='c', flag=True, help='show checkboxes')
	@args.parameter(name='shownotes', short='n', flag=True, help='show text notes')
	@args.parameter(name='blackAndWhite', short='b', flag=True, help='black and white')
	def cod2text(self,
		file,
		checkboxes=False,
		shownotes=False,
		blackAndWhite=False
	):
		'''
		load a cod file and display
		'''
		#self.reorganize(file)
		
		if blackAndWhite:
			for key in list(self.xcolours.keys()):
				self.xcolours[key] = ''
			for key in list(self.xfonts.keys()):
				self.xfonts[key] = ''

		cod = XML(*getContextFromFile(os.path.expanduser(file)))
		self.__convert(file, cod, checkboxes=checkboxes, shownotes=shownotes)


#____________________________________________________________
if __name__ == '__main__': args.execute()





# ./binding.py
# -*- coding: utf-8 -*-
# PyXB bindings for NM:e92452c8d3e28a9e27abfc9994d2007779e7f4c9
# Generated 2020-06-19 16:18:58.998406 by PyXB version 1.2.6 using Python 3.8.2.final.0
# Namespace AbsentNamespace0

from __future__ import unicode_literals
import pyxb
import pyxb.binding
import pyxb.binding.saxer
import io
import pyxb.utils.utility
import pyxb.utils.domutils
import sys
import pyxb.utils.six as _six
# Unique identifier for bindings created at the same time
_GenerationUID = pyxb.utils.utility.UniqueIdentifier('urn:uuid:c2f8caa0-b1f4-11ea-b5cf-80e65012546e')

# Version of PyXB used to generate the bindings
_PyXBVersion = '1.2.6'
# Generated bindings are not compatible across PyXB versions
if pyxb.__version__ != _PyXBVersion:
    raise pyxb.PyXBVersionError(_PyXBVersion)

# A holder for module-level binding classes so we can access them from
# inside class definitions where property names may conflict.
_module_typeBindings = pyxb.utils.utility.Object()

# Import bindings for namespaces imported into schema
import pyxb.binding.datatypes

# NOTE: All namespace declarations are reserved within the binding
Namespace = pyxb.namespace.CreateAbsentNamespace()
Namespace.configureCategories(['typeBinding', 'elementBinding'])

def CreateFromDocument (xml_text, default_namespace=None, location_base=None):
    """Parse the given XML and use the document element to create a
    Python instance.

    @param xml_text An XML document.  This should be data (Python 2
    str or Python 3 bytes), or a text (Python 2 unicode or Python 3
    str) in the L{pyxb._InputEncoding} encoding.

    @keyword default_namespace The L{pyxb.Namespace} instance to use as the
    default namespace where there is no default namespace in scope.
    If unspecified or C{None}, the namespace of the module containing
    this function will be used.

    @keyword location_base: An object to be recorded as the base of all
    L{pyxb.utils.utility.Location} instances associated with events and
    objects handled by the parser.  You might pass the URI from which
    the document was obtained.
    """

    if pyxb.XMLStyle_saxer != pyxb._XMLStyle:
        dom = pyxb.utils.domutils.StringToDOM(xml_text)
        return CreateFromDOM(dom.documentElement, default_namespace=default_namespace)
    if default_namespace is None:
        default_namespace = Namespace.fallbackNamespace()
    saxer = pyxb.binding.saxer.make_parser(fallback_namespace=default_namespace, location_base=location_base)
    handler = saxer.getContentHandler()
    xmld = xml_text
    if isinstance(xmld, _six.text_type):
        xmld = xmld.encode(pyxb._InputEncoding)
    saxer.parse(io.BytesIO(xmld))
    instance = handler.rootObject()
    return instance

def CreateFromDOM (node, default_namespace=None):
    """Create a Python instance from the given DOM node.
    The node tag must correspond to an element declaration in this module.

    @deprecated: Forcing use of DOM interface is unnecessary; use L{CreateFromDocument}."""
    if default_namespace is None:
        default_namespace = Namespace.fallbackNamespace()
    return pyxb.binding.basis.element.AnyCreateFromDOM(node, default_namespace)


# Atomic simple type: RFC822Date
class RFC822Date (pyxb.binding.datatypes.string):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'RFC822Date')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 68, 4)
    _Documentation = None
RFC822Date._CF_pattern = pyxb.binding.facets.CF_pattern()
RFC822Date._CF_pattern.addPattern(pattern='((Mon|Tue|Wed|Thu|Fri|Sat|Sun),\\s*)?\\d\\d?\\s+(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\\s+\\d\\d(\\d\\d)?\\s+\\d\\d:\\d\\d(:\\d\\d)?\\s+([+\\-]?\\d\\d\\d\\d|[A-Z]{2,3})')
RFC822Date._InitializeFacetMap(RFC822Date._CF_pattern)
Namespace.addCategoryObject('typeBinding', 'RFC822Date', RFC822Date)
_module_typeBindings.RFC822Date = RFC822Date

# Atomic simple type: EmailAddress
class EmailAddress (pyxb.binding.datatypes.string):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'EmailAddress')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 74, 4)
    _Documentation = None
EmailAddress._CF_pattern = pyxb.binding.facets.CF_pattern()
EmailAddress._CF_pattern.addPattern(pattern='[a-zA-Z0-9_\\-][a-zA-Z0-9_.\\-]*@[a-zA-Z0-9_\\-][a-zA-Z0-9_.\\-]*')
EmailAddress._InitializeFacetMap(EmailAddress._CF_pattern)
Namespace.addCategoryObject('typeBinding', 'EmailAddress', EmailAddress)
_module_typeBindings.EmailAddress = EmailAddress

# Atomic simple type: Unknown
class Unknown (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Unknown')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 84, 4)
    _Documentation = None
Unknown._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=Unknown, enum_prefix=None)
Unknown.unknown = Unknown._CF_enumeration.addEnumeration(unicode_value='unknown', tag='unknown')
Unknown._InitializeFacetMap(Unknown._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'Unknown', Unknown)
_module_typeBindings.Unknown = Unknown

# Atomic simple type: Version
class Version (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):

    """An atomic simple type."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Version')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 90, 4)
    _Documentation = None
Version._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=Version, enum_prefix=None)
Version.n1_0 = Version._CF_enumeration.addEnumeration(unicode_value='1.0', tag='n1_0')
Version.n2_0 = Version._CF_enumeration.addEnumeration(unicode_value='2.0', tag='n2_0')
Version._InitializeFacetMap(Version._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'Version', Version)
_module_typeBindings.Version = Version

# Union simple type: Language
# superclasses pyxb.binding.datatypes.anySimpleType
class Language (pyxb.binding.basis.STD_union):

    """Simple type that is a union of pyxb.binding.datatypes.language, Unknown."""

    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Language')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 80, 4)
    _Documentation = None

    _MemberTypes = ( pyxb.binding.datatypes.language, Unknown, )
Language._CF_pattern = pyxb.binding.facets.CF_pattern()
Language._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=Language)
Language.unknown = 'unknown'                      # originally Unknown.unknown
Language._InitializeFacetMap(Language._CF_pattern,
   Language._CF_enumeration)
Namespace.addCategoryObject('typeBinding', 'Language', Language)
_module_typeBindings.Language = Language

# Complex type Body with content type ELEMENT_ONLY
class Body (pyxb.binding.basis.complexTypeDefinition):
    """Complex type Body with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Body')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 25, 4)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element outline uses Python identifier outline
    __outline = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'outline'), 'outline', '__AbsentNamespace0_Body_outline', True, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 27, 12), )

    
    outline = property(__outline.value, __outline.set, None, None)

    _ElementMap.update({
        __outline.name() : __outline
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Body = Body
Namespace.addCategoryObject('typeBinding', 'Body', Body)


# Complex type Head with content type ELEMENT_ONLY
class Head (pyxb.binding.basis.complexTypeDefinition):
    """Complex type Head with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Head')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 31, 4)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element title uses Python identifier title
    __title = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'title'), 'title', '__AbsentNamespace0_Head_title', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 33, 12), )

    
    title = property(__title.value, __title.set, None, None)

    
    # Element dateCreated uses Python identifier dateCreated
    __dateCreated = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'dateCreated'), 'dateCreated', '__AbsentNamespace0_Head_dateCreated', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 34, 12), )

    
    dateCreated = property(__dateCreated.value, __dateCreated.set, None, None)

    
    # Element dateModified uses Python identifier dateModified
    __dateModified = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'dateModified'), 'dateModified', '__AbsentNamespace0_Head_dateModified', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 35, 12), )

    
    dateModified = property(__dateModified.value, __dateModified.set, None, None)

    
    # Element ownerName uses Python identifier ownerName
    __ownerName = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'ownerName'), 'ownerName', '__AbsentNamespace0_Head_ownerName', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 36, 12), )

    
    ownerName = property(__ownerName.value, __ownerName.set, None, None)

    
    # Element ownerEmail uses Python identifier ownerEmail
    __ownerEmail = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'ownerEmail'), 'ownerEmail', '__AbsentNamespace0_Head_ownerEmail', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 37, 12), )

    
    ownerEmail = property(__ownerEmail.value, __ownerEmail.set, None, None)

    
    # Element ownerId uses Python identifier ownerId
    __ownerId = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'ownerId'), 'ownerId', '__AbsentNamespace0_Head_ownerId', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 38, 12), )

    
    ownerId = property(__ownerId.value, __ownerId.set, None, None)

    
    # Element docs uses Python identifier docs
    __docs = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'docs'), 'docs', '__AbsentNamespace0_Head_docs', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 39, 12), )

    
    docs = property(__docs.value, __docs.set, None, None)

    
    # Element expansionState uses Python identifier expansionState
    __expansionState = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'expansionState'), 'expansionState', '__AbsentNamespace0_Head_expansionState', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 40, 12), )

    
    expansionState = property(__expansionState.value, __expansionState.set, None, None)

    
    # Element vertScrollState uses Python identifier vertScrollState
    __vertScrollState = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'vertScrollState'), 'vertScrollState', '__AbsentNamespace0_Head_vertScrollState', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 41, 12), )

    
    vertScrollState = property(__vertScrollState.value, __vertScrollState.set, None, None)

    
    # Element windowTop uses Python identifier windowTop
    __windowTop = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'windowTop'), 'windowTop', '__AbsentNamespace0_Head_windowTop', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 42, 12), )

    
    windowTop = property(__windowTop.value, __windowTop.set, None, None)

    
    # Element windowLeft uses Python identifier windowLeft
    __windowLeft = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'windowLeft'), 'windowLeft', '__AbsentNamespace0_Head_windowLeft', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 43, 12), )

    
    windowLeft = property(__windowLeft.value, __windowLeft.set, None, None)

    
    # Element windowBottom uses Python identifier windowBottom
    __windowBottom = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'windowBottom'), 'windowBottom', '__AbsentNamespace0_Head_windowBottom', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 44, 12), )

    
    windowBottom = property(__windowBottom.value, __windowBottom.set, None, None)

    
    # Element windowRight uses Python identifier windowRight
    __windowRight = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'windowRight'), 'windowRight', '__AbsentNamespace0_Head_windowRight', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 45, 12), )

    
    windowRight = property(__windowRight.value, __windowRight.set, None, None)

    _ElementMap.update({
        __title.name() : __title,
        __dateCreated.name() : __dateCreated,
        __dateModified.name() : __dateModified,
        __ownerName.name() : __ownerName,
        __ownerEmail.name() : __ownerEmail,
        __ownerId.name() : __ownerId,
        __docs.name() : __docs,
        __expansionState.name() : __expansionState,
        __vertScrollState.name() : __vertScrollState,
        __windowTop.name() : __windowTop,
        __windowLeft.name() : __windowLeft,
        __windowBottom.name() : __windowBottom,
        __windowRight.name() : __windowRight
    })
    _AttributeMap.update({
        
    })
_module_typeBindings.Head = Head
Namespace.addCategoryObject('typeBinding', 'Head', Head)


# Complex type OPML with content type ELEMENT_ONLY
class OPML (pyxb.binding.basis.complexTypeDefinition):
    """Complex type OPML with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'OPML')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 17, 4)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element head uses Python identifier head
    __head = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'head'), 'head', '__AbsentNamespace0_OPML_head', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 19, 12), )

    
    head = property(__head.value, __head.set, None, None)

    
    # Element body uses Python identifier body
    __body = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'body'), 'body', '__AbsentNamespace0_OPML_body', False, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 20, 12), )

    
    body = property(__body.value, __body.set, None, None)

    
    # Attribute version uses Python identifier version
    __version = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'version'), 'version', '__AbsentNamespace0_OPML_version', _module_typeBindings.Version, required=True)
    __version._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 22, 8)
    __version._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 22, 8)
    
    version = property(__version.value, __version.set, None, None)

    _ElementMap.update({
        __head.name() : __head,
        __body.name() : __body
    })
    _AttributeMap.update({
        __version.name() : __version
    })
_module_typeBindings.OPML = OPML
Namespace.addCategoryObject('typeBinding', 'OPML', OPML)


# Complex type Outline with content type ELEMENT_ONLY
class Outline (pyxb.binding.basis.complexTypeDefinition):
    """Complex type Outline with content type ELEMENT_ONLY"""
    _TypeDefinition = None
    _ContentTypeTag = pyxb.binding.basis.complexTypeDefinition._CT_ELEMENT_ONLY
    _Abstract = False
    _ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Outline')
    _XSDLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 49, 4)
    _ElementMap = {}
    _AttributeMap = {}
    # Base type is pyxb.binding.datatypes.anyType
    
    # Element outline uses Python identifier outline
    __outline = pyxb.binding.content.ElementDeclaration(pyxb.namespace.ExpandedName(None, 'outline'), 'outline', '__AbsentNamespace0_Outline_outline', True, pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 51, 12), )

    
    outline = property(__outline.value, __outline.set, None, None)

    
    # Attribute text uses Python identifier text
    __text = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'text'), 'text', '__AbsentNamespace0_Outline_text', pyxb.binding.datatypes.string, required=True)
    __text._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 53, 8)
    __text._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 53, 8)
    
    text = property(__text.value, __text.set, None, None)

    
    # Attribute isComment uses Python identifier isComment
    __isComment = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'isComment'), 'isComment', '__AbsentNamespace0_Outline_isComment', pyxb.binding.datatypes.boolean, unicode_default='false')
    __isComment._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 54, 8)
    __isComment._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 54, 8)
    
    isComment = property(__isComment.value, __isComment.set, None, None)

    
    # Attribute isBreakpoint uses Python identifier isBreakpoint
    __isBreakpoint = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'isBreakpoint'), 'isBreakpoint', '__AbsentNamespace0_Outline_isBreakpoint', pyxb.binding.datatypes.boolean, unicode_default='false')
    __isBreakpoint._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 55, 8)
    __isBreakpoint._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 55, 8)
    
    isBreakpoint = property(__isBreakpoint.value, __isBreakpoint.set, None, None)

    
    # Attribute created uses Python identifier created
    __created = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'created'), 'created', '__AbsentNamespace0_Outline_created', _module_typeBindings.RFC822Date)
    __created._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 56, 8)
    __created._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 56, 8)
    
    created = property(__created.value, __created.set, None, None)

    
    # Attribute category uses Python identifier category
    __category = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'category'), 'category', '__AbsentNamespace0_Outline_category', pyxb.binding.datatypes.string)
    __category._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 57, 8)
    __category._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 57, 8)
    
    category = property(__category.value, __category.set, None, None)

    
    # Attribute description uses Python identifier description
    __description = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'description'), 'description', '__AbsentNamespace0_Outline_description', pyxb.binding.datatypes.string)
    __description._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 58, 8)
    __description._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 58, 8)
    
    description = property(__description.value, __description.set, None, None)

    
    # Attribute url uses Python identifier url
    __url = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'url'), 'url', '__AbsentNamespace0_Outline_url', pyxb.binding.datatypes.anyURI)
    __url._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 59, 8)
    __url._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 59, 8)
    
    url = property(__url.value, __url.set, None, None)

    
    # Attribute htmlUrl uses Python identifier htmlUrl
    __htmlUrl = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'htmlUrl'), 'htmlUrl', '__AbsentNamespace0_Outline_htmlUrl', pyxb.binding.datatypes.anyURI)
    __htmlUrl._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 60, 8)
    __htmlUrl._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 60, 8)
    
    htmlUrl = property(__htmlUrl.value, __htmlUrl.set, None, None)

    
    # Attribute xmlUrl uses Python identifier xmlUrl
    __xmlUrl = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'xmlUrl'), 'xmlUrl', '__AbsentNamespace0_Outline_xmlUrl', pyxb.binding.datatypes.anyURI)
    __xmlUrl._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 61, 8)
    __xmlUrl._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 61, 8)
    
    xmlUrl = property(__xmlUrl.value, __xmlUrl.set, None, None)

    
    # Attribute title uses Python identifier title
    __title = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'title'), 'title', '__AbsentNamespace0_Outline_title', pyxb.binding.datatypes.string)
    __title._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 62, 8)
    __title._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 62, 8)
    
    title = property(__title.value, __title.set, None, None)

    
    # Attribute version uses Python identifier version
    __version = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'version'), 'version', '__AbsentNamespace0_Outline_version', pyxb.binding.datatypes.string)
    __version._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 63, 8)
    __version._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 63, 8)
    
    version = property(__version.value, __version.set, None, None)

    
    # Attribute language uses Python identifier language
    __language = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'language'), 'language', '__AbsentNamespace0_Outline_language', _module_typeBindings.Language)
    __language._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 64, 8)
    __language._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 64, 8)
    
    language = property(__language.value, __language.set, None, None)

    
    # Attribute type uses Python identifier type
    __type = pyxb.binding.content.AttributeUse(pyxb.namespace.ExpandedName(None, 'type'), 'type', '__AbsentNamespace0_Outline_type', pyxb.binding.datatypes.string)
    __type._DeclarationLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 65, 8)
    __type._UseLocation = pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 65, 8)
    
    type = property(__type.value, __type.set, None, None)

    _ElementMap.update({
        __outline.name() : __outline
    })
    _AttributeMap.update({
        __text.name() : __text,
        __isComment.name() : __isComment,
        __isBreakpoint.name() : __isBreakpoint,
        __created.name() : __created,
        __category.name() : __category,
        __description.name() : __description,
        __url.name() : __url,
        __htmlUrl.name() : __htmlUrl,
        __xmlUrl.name() : __xmlUrl,
        __title.name() : __title,
        __version.name() : __version,
        __language.name() : __language,
        __type.name() : __type
    })
_module_typeBindings.Outline = Outline
Namespace.addCategoryObject('typeBinding', 'Outline', Outline)


opml = pyxb.binding.basis.element(pyxb.namespace.ExpandedName(Namespace, 'opml'), OPML, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 15, 4))
Namespace.addCategoryObject('elementBinding', opml.name().localName(), opml)



Body._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'outline'), Outline, scope=Body, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 27, 12)))

def _BuildAutomaton ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton
    del _BuildAutomaton
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(Body._UseForTag(pyxb.namespace.ExpandedName(None, 'outline')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 27, 12))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
         ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
Body._Automaton = _BuildAutomaton()




Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'title'), pyxb.binding.datatypes.string, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 33, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'dateCreated'), RFC822Date, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 34, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'dateModified'), RFC822Date, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 35, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'ownerName'), pyxb.binding.datatypes.string, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 36, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'ownerEmail'), EmailAddress, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 37, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'ownerId'), pyxb.binding.datatypes.anyURI, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 38, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'docs'), pyxb.binding.datatypes.anyURI, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 39, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'expansionState'), pyxb.binding.datatypes.string, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 40, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'vertScrollState'), pyxb.binding.datatypes.positiveInteger, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 41, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'windowTop'), pyxb.binding.datatypes.integer, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 42, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'windowLeft'), pyxb.binding.datatypes.integer, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 43, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'windowBottom'), pyxb.binding.datatypes.integer, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 44, 12)))

Head._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'windowRight'), pyxb.binding.datatypes.integer, scope=Head, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 45, 12)))

def _BuildAutomaton_ ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_
    del _BuildAutomaton_
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 33, 12))
    counters.add(cc_0)
    cc_1 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 34, 12))
    counters.add(cc_1)
    cc_2 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 35, 12))
    counters.add(cc_2)
    cc_3 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 36, 12))
    counters.add(cc_3)
    cc_4 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 37, 12))
    counters.add(cc_4)
    cc_5 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 38, 12))
    counters.add(cc_5)
    cc_6 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 39, 12))
    counters.add(cc_6)
    cc_7 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 40, 12))
    counters.add(cc_7)
    cc_8 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 41, 12))
    counters.add(cc_8)
    cc_9 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 42, 12))
    counters.add(cc_9)
    cc_10 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 43, 12))
    counters.add(cc_10)
    cc_11 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 44, 12))
    counters.add(cc_11)
    cc_12 = fac.CounterCondition(min=0, max=1, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 45, 12))
    counters.add(cc_12)
    states = []
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'title')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 33, 12))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_1, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'dateCreated')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 34, 12))
    st_1 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_2, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'dateModified')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 35, 12))
    st_2 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_2)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_3, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'ownerName')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 36, 12))
    st_3 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_3)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_4, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'ownerEmail')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 37, 12))
    st_4 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_4)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_5, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'ownerId')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 38, 12))
    st_5 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_5)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_6, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'docs')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 39, 12))
    st_6 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_6)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_7, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'expansionState')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 40, 12))
    st_7 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_7)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_8, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'vertScrollState')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 41, 12))
    st_8 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_8)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_9, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'windowTop')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 42, 12))
    st_9 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_9)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_10, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'windowLeft')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 43, 12))
    st_10 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_10)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_11, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'windowBottom')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 44, 12))
    st_11 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_11)
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_12, False))
    symbol = pyxb.binding.content.ElementUse(Head._UseForTag(pyxb.namespace.ExpandedName(None, 'windowRight')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 45, 12))
    st_12 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_12)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_2, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_0, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_0, False) ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_1, [
        fac.UpdateInstruction(cc_1, True) ]))
    transitions.append(fac.Transition(st_2, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_1, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_1, False) ]))
    st_1._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_2, [
        fac.UpdateInstruction(cc_2, True) ]))
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_2, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_2, False) ]))
    st_2._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_3, [
        fac.UpdateInstruction(cc_3, True) ]))
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_3, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_3, False) ]))
    st_3._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_4, [
        fac.UpdateInstruction(cc_4, True) ]))
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_4, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_4, False) ]))
    st_4._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_5, [
        fac.UpdateInstruction(cc_5, True) ]))
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_5, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_5, False) ]))
    st_5._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_6, [
        fac.UpdateInstruction(cc_6, True) ]))
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_6, False) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_6, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_6, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_6, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_6, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_6, False) ]))
    st_6._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_7, [
        fac.UpdateInstruction(cc_7, True) ]))
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_7, False) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_7, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_7, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_7, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_7, False) ]))
    st_7._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_8, [
        fac.UpdateInstruction(cc_8, True) ]))
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_8, False) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_8, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_8, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_8, False) ]))
    st_8._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_9, [
        fac.UpdateInstruction(cc_9, True) ]))
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_9, False) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_9, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_9, False) ]))
    st_9._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_10, [
        fac.UpdateInstruction(cc_10, True) ]))
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_10, False) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_10, False) ]))
    st_10._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_11, [
        fac.UpdateInstruction(cc_11, True) ]))
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_11, False) ]))
    st_11._set_transitionSet(transitions)
    transitions = []
    transitions.append(fac.Transition(st_12, [
        fac.UpdateInstruction(cc_12, True) ]))
    st_12._set_transitionSet(transitions)
    return fac.Automaton(states, counters, True, containing_state=None)
Head._Automaton = _BuildAutomaton_()




OPML._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'head'), Head, scope=OPML, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 19, 12)))

OPML._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'body'), Body, scope=OPML, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 20, 12)))

def _BuildAutomaton_2 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_2
    del _BuildAutomaton_2
    import pyxb.utils.fac as fac

    counters = set()
    states = []
    final_update = None
    symbol = pyxb.binding.content.ElementUse(OPML._UseForTag(pyxb.namespace.ExpandedName(None, 'head')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 19, 12))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    final_update = set()
    symbol = pyxb.binding.content.ElementUse(OPML._UseForTag(pyxb.namespace.ExpandedName(None, 'body')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 20, 12))
    st_1 = fac.State(symbol, is_initial=False, final_update=final_update, is_unordered_catenation=False)
    states.append(st_1)
    transitions = []
    transitions.append(fac.Transition(st_1, [
         ]))
    st_0._set_transitionSet(transitions)
    transitions = []
    st_1._set_transitionSet(transitions)
    return fac.Automaton(states, counters, False, containing_state=None)
OPML._Automaton = _BuildAutomaton_2()




Outline._AddElement(pyxb.binding.basis.element(pyxb.namespace.ExpandedName(None, 'outline'), Outline, scope=Outline, location=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 51, 12)))

def _BuildAutomaton_3 ():
    # Remove this helper function from the namespace after it is invoked
    global _BuildAutomaton_3
    del _BuildAutomaton_3
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=None, metadata=pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 51, 12))
    counters.add(cc_0)
    states = []
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = pyxb.binding.content.ElementUse(Outline._UseForTag(pyxb.namespace.ExpandedName(None, 'outline')), pyxb.utils.utility.Location('/Users/dave/git/github.com/Public/Swapsies/xsd/opml.xsd', 51, 12))
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.append(st_0)
    transitions = []
    transitions.append(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, True, containing_state=None)
Outline._Automaton = _BuildAutomaton_3()




#!/usr/bin/env python3

################################################################################
cdata2xmlTokens = [
	{
		'&amp;lt;': '%%%%lt%%%%'
	},
	{
		'&amp;gt;': '%%%%gt%%%%'
	},
	{
		'&amp;amp;': '%%%%amp%%%%'
	},
	{
		'&amp;quot;': '%%%%quot%%%%'
	},
	{
		'&amp;apos;': '%%%%apos%%%%'
	},
	{
		'&lt;': '<'
	},
	{
		'&gt;': '>'
	},
	{
		'&quot;': '\"'
	},
	{
		'&apos;': '\''
	},
	{
		'&nbsp;': ' '
	},
	{
		'&#10;': '\n'
	},
	{
		'&amp;': '&'
	},
	{
		'%%%%lt%%%%': '&lt;'
	},
	{
		'%%%%gt%%%%': '&gt;'
	},
	{
		'%%%%amp%%%%': '%amp;'
	},
	{
		'%%%%quot%%%%': '%quot;'
	},
	{
		'%%%%apos%%%%': '%apos;'
	},
]


def cdata2xmlTokenize(line):
	"""Convert normal xml tags to escaped xml tags"""
	for d in cdata2xmlTokens:
		key = list(d.keys())[0]
		val = d[key]
		while key in line:
			line = line.replace(key, val)
	return line


def cdata2xml(input, output):
	"""Convert normal xml tags to escaped xml tags"""
	for line in input.read().split('\n'):
		output.write(cdata2xmlTokenize(line))
		output.flush()


################################################################################
xml2cdataTokens = [
	{
		'&': '%%%%amp%%%%'
	},
	{
		'<': '&lt;'
	},
	{
		'>': '&gt;'
	},
	{
		'\"': '&quot;'
	},
	{
		'\'': '&apos;'
	},
	{
		'\n': '&#10;'
	},
	{
		'%%%%amp%%%%': '&amp;'
	},
]


def xml2cdataTokenize(line):
	"""Convert xml escaped tags to normal xml inside cdata"""
	for d in xml2cdataTokens:
		key = list(d.keys())[0]
		val = d[key]
		while key in line:
			line = line.replace(key, val)
	return line


def xml2cdata(input, output):
	"""Convert xml escaped tags to normal xml inside cdata"""
	for line in input.read().split('\n'):
		output.write(xml2cdataTokenize(line))
		output.flush()





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

####################################################################################
import sys, re, os, io, argparse, argcomplete
sys.path.insert(0,'..')

from subprocess import Popen, PIPE

from Swapsies.xmi import *
from GoldenChild.xpath import *
from Perdy.parser import *
from Perdy.pretty import *
from McUnix.finder import *

####################################################################################
parser = argparse.ArgumentParser()

fgroup = parser.add_mutually_exclusive_group()
fgroup.add_argument('-l', '--link', action='store_true', help='link local files')
fgroup.add_argument('-s', '--svn', action='store_true', help='link svn files')

parser.add_argument('-v', '--verbose', action='store_true', help='show help')
parser.add_argument('-c', '--cygwin', action='store_true', help='shell is cygwin')
parser.add_argument('-f', '--files', action='store_true', help='included files')
parser.add_argument('-o', '--output', action='store', help='output.xmi file name', default='.xmi')
parser.add_argument('dir', action='store', help='the directory')

argcomplete.autocomplete(parser)
args = parser.parse_args()

####################################################################################
if args.verbose:
	prettyPrint(vars(args), colour=True, output=sys.stderr)
	sys.stderr.flush()

if args.svn:
	process = Popen('svn info --xml %s' % args.dir, shell=True, stdout=PIPE)
	(sdoc, sctx) = getContextFromString(
		''.join([x.rstrip() for x in process.stdout.readlines()]))
	del process
	svn = getElementText(sctx, '/info/entry/url')
	if args.verbose:
		sys.stderr.write('svn=%s\n' % svn)
		sys.stderr.flush()


####################################################################################
def concatPath(prefix, suffix):
	path = suffix
	if prefix:
		path = '%s/%s' % (prefix.rstrip('/'), suffix)
	return path


####################################################################################
class DirectoryTree:
	@property
	def doc(self):
		return self.xmi.doc

	def __init__(self, xmi=XMI()):
		self.xmi = xmi

	def makeDirectory(self, name, parent=None, prefix=None):
		if parent == None:
			parent = self.xmi.modelNS
		element = self.xmi.makePackage(name, parent)
		dir = concatPath(prefix, name)
		for f in os.listdir(dir):
			fp = concatPath(dir, f)
			if args.verbose:
				sys.stderr.write('< %s\n' % fp)
				sys.stderr.flush()
			if os.path.isfile(fp) and args.files:
				self.makeFile(f, element, dir)
			elif os.path.isdir(fp):
				self.makeDirectory(f, element, dir)
		return

	def makeFile(self, name, parent, prefix=None):
		classCF = self.xmi.makeClass(name, parent)
		fp = os.path.abspath(concatPath(prefix, name))
		if args.cygwin:
			p = re.compile('^/cygdrive/([a-z])(/.*)$')
			m = p.match(fp)
			if m:
				fp = '%s:%s' % m.groups()
			if fp[0] == '/':
				fp = 'c:/cygwin%s' % fp
			fp = fp.replace('/', '\\')
		self.xmi.makeEAFile(fp, 'Local File', classCF.parent)
		return


####################################################################################
def main():
	global files

	dt = DirectoryTree()
	dt.makeDirectory(args.dir)

	if args.output:
		sys.stderr.write('> %s\n' % args.output)
		output = open(args.output, 'w')
	else:
		output = sys.stdout

	input = io.StringIO('%s\n' % dt.doc)
	doParse(input, output)
	input.close()
	output.close()
	return


if __name__ == '__main__': main()





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, re, xmltodict, argparse, argcomplete, json, vobject, datetime

parser = argparse.ArgumentParser('Convert microsoft .contact to .vcard')

parser.add_argument(
	'-v', '--verbose', action='store_true', help='show detailed output')
parser.add_argument(
	'-o', '--output', action='store', help='output dir', default='vcards')
parser.add_argument(
	'emails',
	action='store',
	help='file containing comma seperated email addresses')

argcomplete.autocomplete(parser)
args = parser.parse_args()

companies = {
	'tradelink.com.au': 'Tradelink',
	'fbu.com': 'Fletcher Building',
	'stibo.com.au': 'Stibo Systems Australia',
	'stibo.com': 'Stibo Systems',
}

if args.verbose:
	sys.stderr.write('args:')
	json.dump(vars(args), sys.stderr, indent=4)
	sys.stderr.write('\n')


def convert(fqn):
	fqn = fqn.strip()
	triangles = re.compile('^([^<]+) <([^>]+)>$')
	match = triangles.match(fqn)
	if match:
		names = match.group(1).split(' ')
		email = match.group(2)
	else:
		email = fqn
		rawemail = re.compile('^([^@]+)@(.*)')
		match = rawemail.match(fqn)
		if match:
			names = match.group(1).split('.')
		else:
			names = ''

	vcard = vobject.vCard()
	vcard.add('n')

	for n in range(len(names)):
		name = names[n - 1]
		if len(name) > 1:
			names[n - 1] = name[0].upper() + name[1:]
		elif len(name) > 0:
			names[n - 1] = name[0].upper()

	if len(names) == 1:
		vcard.n.value = vobject.vcard.Name(
			given=names[0],
			additional='',
			family='', )
	if len(names) == 2:
		vcard.n.value = vobject.vcard.Name(
			given=names[0],
			additional='',
			family=names[1], )
	if len(names) == 3:
		vcard.n.value = vobject.vcard.Name(
			given=names[0], additional=names[1], family=names[2])
	if len(names) > 3:
		sys.stderr.write('error names=%s\n' % names)

	vcard.add('fn')
	vcard.fn.value = ' '.join(names)

	domains = re.compile('^([^@]+)@(.*)$')
	match = domains.match(email)
	if not match:
		sys.stderr.write('domains failed=%s' % email)
		return

	company = match.group(2)

	if company in list(companies.keys()):
		company = '%s' % companies[company]

	vcard.add('org')
	vcard.org.value = [company]

	ve = vcard.add('email')
	ve.value = email

	vcard.prettyPrint()

	target = '%s.vcf' % ' '.join(names)
	if args.output:
		target = '%s/%s' % (args.output, target)

	output = open(target, 'w')
	vcard.serialize(buf=output)
	output.close()
	return


def main():
	if args.output and not os.path.isdir(args.output):
		os.makedirs(args.output)

	input = open(args.emails)
	for email in input.readlines():
		convert(email)
	input.close()

	return


if __name__ == '__main__': main()





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, re, json, argparse, argcomplete, logging

sys.path.append('..')

from Swapsies.cdata import *


def argue():
	parser = argparse.ArgumentParser()

	parser.add_argument('-v', '--verbose', action='store_true')
	parser.add_argument('-r', '--reverse', action='store_true')
	parser.add_argument('-w', '--wrap', action='store_true')
	parser.add_argument('-i', '--input', action='store')
	parser.add_argument('-o', '--output', action='store')

	argcomplete.autocomplete(parser)
	args = parser.parse_args()
	if args.verbose:
		sys.stderr.write('%s\n' % json.dumps(vars(args), indent=4))
	return args


def main():
	args = argue()

	level = logging.INFO
	if args.verbose:
		level = logging.DEBUG

	logging.basicConfig(level=level)

	input = sys.stdin
	if args.input:
		input = open(args.input)

	output = sys.stdout
	if args.output:
		output = open(args.output, 'w')

	if args.reverse:
		if args.wrap:
			output.write('<xml attr="')
		xml2cdata(input, output)
		if args.wrap:
			output.write('"/>')
	else:
		cdata2xml(input, output)

	if args.output:
		output.close()
	if args.input:
		input.close()

	return


if __name__ == '__main__': main()




#!/usr/bin/env python3

import os,re,sys

from GoldenChild.xpath import *
from Perdy.parser import printXML

file=sys.argv[1]

(doc,ctx,nsp) = getContextFromFile(file)

ctx.xpathRegisterNs('xs',"http://www.w3.org/2001/XMLSchema")

elements = getElements(ctx,'/xs:schema/xs:element[@name]')
for element in elements:
	if getAttribute(element, 'ref'): continue
	name = getAttribute(element,'name')

	tipe = getAttribute(element, 'type')
	if tipe:
		st = addElement(doc, 'simpleType')
		setAttribute(st, 'name', name)
		rs = addElement(doc, 'restriction', st)
		setAttribute(rs, 'base', tipe)
	else:
		ct = getElement(ctx,'xs:complexType',element)
		if not ct: continue
		ct.unlinkNode()
		setAttribute(ct, 'name', name)
		doc.getRootElement().addChild(ct)

	element.unlinkNode()
	element.freeNode()

elements = getElements(ctx,'//xs:element[@ref]')
for element in elements:
	if getAttribute(element,'name'): continue
	
	ref = getAttribute(element, 'ref')

	tipe = getElement(ctx, '/xs:schema/xs:complexType[@name="%s"]'%ref) \
		or getElement(ctx, '/xs:schema/xs:simpleType[@name="%s"]'%ref)

	property = getElement(ctx, '@ref', element)
	property.unlinkNode()
	property.freeNode()
	
	setAttribute(element, 'name', ref)
	setAttribute(element, 'type', ref)

with open(file.replace('.xsd','.types.xsd'),'w') as output:
	root_name = sys.argv[2]
	root = addElement(doc, 'element')
	setAttribute(root, 'name', root_name)
	setAttribute(root, 'type', root_name)
	printXML(str(doc),output=output)


